#include "testApp.h"


// ----  AR TK+ STUFF - ripped from the multi marker demo app
// needs a home in a nice ofxARToolkit addon :)

#include "ARToolKitPlus/TrackerMultiMarkerImpl.h"


class MyLogger : public ARToolKitPlus::Logger
{
    void artLog(const char* nStr)
    {
        printf(nStr);
    }
};

static const int       width = 720, height = 576, bpp = 1;
static   size_t        numPixels = width*height*bpp;
//static    size_t        numBytesRead;
//static   const char    *fName = "data/markerboard_480-499.raw";
static    unsigned char *cameraBuffer = new unsigned char[numPixels];
static    MyLogger      logger;
static bool useBCH = false;


ARToolKitPlus::TrackerMultiMarker *tracker;

static bool bDraw = false;




//--------------------------------------------------------------
void testApp::setup(){	
	
	ofSetVerticalSync(true);
	ofSetFrameRate(60);
	
	grabber.initGrabber(width, height);
	convert.allocate(grabber.getWidth(), grabber.getHeight());
	gray.allocate(grabber.getWidth(), grabber.getHeight());
	grayMirror.allocate(grabber.getWidth(), grabber.getHeight());
	videoTexture.allocate(grabber.getWidth(), grabber.getHeight(), GL_LUMINANCE);	
	
	ofBackground(127,127,127);
	
	//initialisation de l'ensemble des touches et de leurs matrices
	for(int i=0; i<24; i++){
		//touches[i]=false;
		touches_compteur[i]=0;
		touches_jeu[i]=false;
		for (int j=0; j<16; j++){
			mView[i][j] = 0;
		}
	}
	//initialisation des variables permettant les processus de recalage
	decCentreX = 0;
	centerPoint = false;
	
	// open an outgoing connection to HOST:PORT
	sender.setup( HOST, PORT );
	
	//definition de l'interface et activation et sauvegarde
	gui.addSlider("Threshold", Threshold, 0, 255);
	gui.addSlider("Speed limit", speedlimit, 0, 100);
	gui.addToggle("View number of marker", view_number);
	gui.addToggle("Miror", miror);
	//gui.addFPSCounter();
	gui.loadFromXML();
	gui.show();
	
	
	idNumb.loadFont("Arial.ttf", 10);	
	
	// ----------------------------------  AR TK+ STUFF - ripped from the single marker demo app
	
    // create a tracker that does:
    //  - 6x6 sized marker images
    //  - samples at a maximum of 6x6
    //  - works with luminance (gray) images
    //  - can load a maximum of 10 patterns
    //  - can detect a maximum of 10 patterns in one image
	tracker = new ARToolKitPlus::TrackerMultiMarkerImpl<6,6,6, 10, 10>(width,height);
	
	const char* description = tracker->getDescription();
	printf("ARToolKitPlus compile-time information:\n%s\n\n", description);
	
    // set a logger so we can output error messages
    tracker->setLogger(&logger);
	tracker->setPixelFormat(ARToolKitPlus::PIXEL_FORMAT_LUM);	
	
    if( !tracker->init( (const char *)ofToDataPath("LogitechPro4000.dat").c_str(), (const char *)ofToDataPath("markerboard_480-499.cfg").c_str(), 1.0f, 1000.0f) )            // load std. ARToolKit camera file
	{
		printf("ERROR: init() failed\n");
		delete cameraBuffer;
		delete tracker;
		return;
	}
	
    // define size of the marker
    //tracker->setPatternWidth(80); // I'm not sure how to define the size with multimarkers since it doesnt seem to have this option.
	
	// the marker in the BCH test image has a thin border...
    tracker->setBorderWidth(useBCH ? 0.125f : 0.250f);
	
    // set a threshold. alternatively we could also activate automatic thresholding
    //tracker->setThreshold(Threshold);
	tracker->activateAutoThreshold(true);
	
    // let's use lookup-table undistortion for high-speed
    // note: LUT only works with images up to 1024x1024
    tracker->setUndistortionMode(ARToolKitPlus::UNDIST_LUT);
	
    // RPP is more robust than ARToolKit's standard pose estimator
    tracker->setPoseEstimator(ARToolKitPlus::POSE_ESTIMATOR_RPP);
	
    // switch to simple ID based markers
    // use the tool in tools/IdPatGen to generate markers
    tracker->setMarkerMode(useBCH ? ARToolKitPlus::MARKER_ID_BCH : ARToolKitPlus::MARKER_ID_SIMPLE);
}

//--------------------------------------------------------------
void testApp::update(){
	
	
	grabber.grabFrame();
	if(grabber.isFrameNew())
	{
		
		//convert our camera frame to grayscale
		convert.setFromPixels(grabber.getPixels(), grabber.getWidth(), grabber.getHeight());
		gray = convert;
		gray.threshold(Threshold);
		//u dilate et un erode semble ce qui permet d'ouvrir le plus possible le champ de la camera
		//gray.dilate();
		//gray.erode();
		
		//find the marker and get back the confidence
		int markerId = tracker->calc(gray.getPixels());
		
		if (miror){
			gray.mirror(false, true);
			grayMirror = gray;
		}
		else grayMirror = gray;
		
		unsigned char * seuil = grayMirror.getPixels();
		videoTexture.loadData(seuil, grabber.getWidth(), grabber.getHeight(), GL_LUMINANCE);
		
		
		
	}
	
//________________________________________________________
	//  On efface tout l'ensemble des presences de marqueur avant de le renseigner
	
	for(int i=0; i<24; i++){
		touches[i]=false;
		}

	//this is where we use the calculated matrix from ARToolkitPlus to put
	//in our graphics at the location and orientation of the marker.
	//- the matrix has the 0,0 point as the center of the marker. 
	
	int numDetected = tracker->getNumDetectedMarkers();
	//cout<<"numberofmarkers:"<<numDetected<<endl;
	
//__________________________________________________________
	//on parcourt les marqueurs pour trouver les matrices
	
	for(int i=0; i<numDetected; i++)
	{
		//On recupere le marker
		ARToolKitPlus::ARMarkerInfo marker = tracker->getDetectedMarker(i);
		//on alimente l'ensemble principal
		touches[marker.id] = true;
		
		//on definit la matrice, vecteur, valeur permettant de construire la modelview matrice
		float m34[ 3 ][ 4 ];
		float c[ 2 ] = { 0.0f, 0.0f };
		float w = 40.0f;
		
		//on recupere ces valeurs
		tracker->rppGetTransMat( &marker, c, w, m34 );
				
		//on construit la model view matrice
		 //this is some crazy matrix transformative stuff.  Got it from here: http://chihara.naist.jp/people/STAFF/imura/computer/OpenGL/artp/disp_content
		//la matrice 3 lignes, 4 colonnes est completee de 0 pour la definition du referentiel et de 1 en scalar pour la translation
		for ( int i = 0; i < 3; i++ ) {
			for ( int j = 0; j < 4; j++ ) {
				mView[marker.id][ j * 4 + i ] = m34[ i ][ j ];
				//matrixView[marker.id](i, j ) = m34[ i ][ j ];
			}
		}
		for ( int j = 0; j < 3; j++ ) {
			mView[marker.id][ j * 4 + 3 ] = 0.0f;
			//matrixView[marker.id](j, 3 ) = 0.0f;
		}
		mView[marker.id][ 3 * 4 + 3 ] = 1.0f;
		//matrixView[marker.id](3, 3 ) = 0.0f;
		
		//on recupere les valeurs de milieu 
		if (centerPoint){
			decCentreX = mView[marker.id][12];
			centerPoint = false;
		}
	}
		
//___________________________________________________________
	//boucle permettant de filtrer les disparitions fugaces dues a la qualite de la detection
	//et faire les envois Osc
	
	for(int i=0; i<24; i++){
		//on calcule l'angle de l'axe x
		float angle[24];
		if (touches[i] == true){
				if((mView[i][0]>0) && (mView[i][1]>0)){angle[i] = atan(mView[i][1]/mView[i][0])*RAD_TO_DEG;}
				if(mView[i][0]<0){angle[i] = 180 + atan(mView[i][1]/mView[i][0])*RAD_TO_DEG;}
				if((mView[i][0]>0) && (mView[i][1]<0)){angle[i] = 360 + atan(mView[i][1]/mView[i][0])*RAD_TO_DEG;}
		}
		//on regarde la valeur de la note
		if (touches[i] == false){
			//on regarde si la note precedente est note on et 
			//si le compteur de speedlimit est en dessous de 0
			if ((touches_jeu[i]==true) && (touches_compteur[i]<0)){
				//si oui on envoie la note off via Osc
				ofxOscMessage m;
				m.setAddress( "/Note" );
				m.addIntArg( i );
				m.addIntArg( 0 );
				sender.sendMessage( m );
				touches_jeu[i]=false;
			}
			else{
				//sinon on baisse le compteur 
				if (touches_jeu[i]==true){
					touches_compteur[i] = touches_compteur[i] - 1;
					//on envoie la note on via Osc
					ofxOscMessage m;
					m.setAddress( "/Note" );
					m.addIntArg( i );
					m.addIntArg( 127 );
					//m.addFloatArg(angle[i]);
					sender.sendMessage( m );
				}
			}
		}
		else{
			if (touches_jeu[i]==false){
				//on envoie la note on via Osc
				ofxOscMessage m;
				m.setAddress( "/Note" );
				m.addIntArg( i );
				m.addIntArg( 127 );
				//m.addFloatArg(angle[i]);
				sender.sendMessage( m );
				
				touches_jeu[i]=true;
				touches_compteur[i] = speedlimit;
			}
			else {
				touches_compteur[i] = speedlimit;
				ofxOscMessage m;
				m.setAddress( "/Note" );
				m.addIntArg( i );
				m.addIntArg( 127 );
				//m.addFloatArg(angle[i]);
				sender.sendMessage( m );
			}
		}
	}
}

//--------------------------------------------------------------
void testApp::draw(){
	
	ofSetColor(255, 255, 255);
	videoTexture.draw(0,0,width, height);
	
	gui.draw();
	
	//Matrice multiplicative permettant de faire la symetrie en miroir
	float smir[ 16 ] = { -1.0f, 1.0f, 1.0f, 1.0f, -1.0f, 1.0f, 1.0f, 1.0f, -1.0f, 1.0f, 1.0f, 1.0f, -1.0f, 1.0f, 1.0f, 1.0f };
	
	glViewport(0, 0, width, height );
	glMatrixMode( GL_PROJECTION );
	glLoadMatrixf(tracker->getProjectionMatrix());
		
	if (view_number){
		for(int i=0; i<24; i++){
			if (touches_jeu[i]==true){
		//on applique des modifications a la matrice si necessaire
				//On recale le centre si besoin 
				if (touches[i]==true){
					mView[i][12] = mView[i][12] - decCentreX;
				}
				
				//on modifie en miroir quand necessaire
				if ((miror==true) && (touches[i]==true)){      
					for(int k=0; k<16; k++){
						mView[i][k] = mView[i][k] * smir[k];
					}
				}
				
		//On integre la modelview matrix du marqueur dans le stack et on dessine
				glMatrixMode( GL_MODELVIEW );
				glLoadMatrixf( mView[i]);
				glPushMatrix();

				/*ofEnableSmoothing();
				ofSetColor(255 , 0, 0 );
				ofSetLineWidth(2);
				ofLine(-2, 0, 10, 0);
				ofSetColor(0, 0, 255 );
				ofLine(0, -2, 0, 10);
				ofDisableSmoothing();*/
				//choix de la couleur pour faire etat de la stabilite 
				if (touches[i]==true){ofSetColor( 0, 255, 0 );}
				else ofSetColor( 255, 0, 0 );
				
				glRotatef(180, 1, 0, 0);
				idNumb.drawString(ofToString(i), -idNumb.stringWidth(ofToString(i))/2, 0);
				
				glPopMatrix();
			}
		}
		
	}
	
}

//-------------------------------------------------------------
void testApp::captureCenter(){
	centerPoint = !centerPoint;
}


//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 

	switch (key) {
		case 'f':
			ofToggleFullscreen();
			break;
		case 'i':
			gui.toggleDraw();
			break;
		case 'v':
			grabber.videoSettings();
			break;
		case 'c':
			captureCenter();
		default:
			break;
	}
	
}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){ 
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mouseReleased(){
	
}
